## S3Search Heroku add-on documentation

The [S3Search add-on](http://addons.heroku.com/s3search) provides powerful document-based full-text indexing and search to your application.

Adding S3Search to your application will allow you search your documents based upon the actual text within the documents, as well as any metadata fields you assign to them. Yes, that's right! S3Search will index the text inside your documents.

Do you already have documents stored and want to index them and make them searchable? No worries. S3Search works by you sending it a URL to fetch the document content to index, along with a hash of metadata attributes to record against it.  You can them perform powerful queries against that indexed data, based on the rich features of [elasticsearch](http://www.elasticsearch.org).

S3Search is accessible via a supported Ruby API. It also has a HTTP API to support other languages.

## Provisioning the add-on

S3Search can be attached to a Heroku application via the  CLI:

>A list of all plans available can be found [here](http://addons.heroku.com/s3search).


````term
    $ heroku addons:add s3search
    -----> Adding s3search to sharp-mountain-4005... done, v18 (free)
````
Once S3Search has been added, three new settings will be added to your app configuration: `S3SEARCH_URL`, `S3SEARCH_API_KEY`, and `S3SEARCH_API_SECRET`. This can be confirmed using the `heroku config:get` command.

````term
    $ heroku config:get S3SEARCH_URL
    https://us-east-1.s3searchapp.com
    $ heroku config:get S3SEARCH_API_KEY
    4ef4499b6911fcf89ccb455a92571cb7
````
After installing S3Search the application should be configured to fully integrate with the add-on.

## Using with Ruby applications

Ruby on Rails applications will need to add the following entry into their `Gemfile` specifying the S3Search client library.

````ruby
    gem 's3search'
````
Update application dependencies with bundler.

````term
    $ bundle install
````

## How it works

S3Search uses a special attribute, `_content_url`, to asynchronously index any content you want to associate with a document.  `_content_url` is an optional field.

When a document is added with a `_content_url` field, it will normally be indexed within a few minutes. Another special field is added to the document, `_is_indexed`, that will be set to true when the indexing of the content located at `_content_url` is complete.

If you want to index content that is in a private S3 Bucket, simply use a temporarily authenticated URL for the `_content_url` value.  The expiration time of this link should be set to no less than 1 hour.

Let's write some application code to index some documents.

````ruby
    S3Search.create title: 'MyDocument', _content_url: 'https://s3-us-east-1.amazonaws.com/my_bucket/my_document.pdf'
    S3Search.create name: 'Bob Lob Law', resume_id: 25, _content_url: 'https://s3-us-east-1.amazonaws.com/resumes.mycompany.com/bob.pdf'
````
The documents don't even really need to be in S3.
    
````ruby
    S3Search.create name: 'Bitcoin Pirate', resume_id: 42, _content_url: 'https://user:pass@authenticatedlocation.com/docs/jenny.pdf'
    S3Search.create title: 'Bitcoin', author: 'santoshin@gmx.com', tags: ['bitcoin', 'manifesto'], _content_url: 'http://bitcoin.org/bitcoin.pdf'
````
The documents don't even really need to be documents! You can use S3Search to use its powerful search capability over just your custom metadata.

````ruby
    S3Search.create customer_id: 32, first_name: 'His Holiness', last_name: 'The Dalia Lama', religion: 'Buddhist', twitter_handle: '@DalaiLama'
    S3Search.create customer_id: 99, first_name: 'George', middle_name: 'R. R.', last_name: 'Martin', job_title: 'Author'
```` 
Now retrieve some documents via the powerful query API.

Search by a single metadata field

````ruby
    results = S3Search.simple_search('title:MyDocument')
````
Search all metadata fields AND the content of the documents.

````ruby
    results = S3Search.simple_search('bitcoin')
```
Search only the content of the documents.

````ruby
    results = S3Search.simplesearch('_document_content:bitcoin')
```` 
 Boost the search ranking of a certain field.
 
````ruby
     results = S3Search.search(where: {_all: 'bitcoin'}, boost: { title: 2.5 })
````
 Find a single document based on its unique id.
 
````ruby
     document = S3Search.get '833FCA4EEEF2943AC2D8E0'
````

## Dashboard

The S3Search dashboard allows you to view the current status of your S3Search cluster.

The dashboard can be accessed via the CLI:

````term
    $ heroku addons:open s3search
    Opening s3search for sharp-mountain-4005...
````
or by visiting the [Heroku apps web interface](http://heroku.com/myapps) and selecting the application in question. Select S3Search from the Add-ons menu.

## Migrating between plans

>Application owners should carefully manage the migration timing to ensure proper application function during the migration process

Use the `heroku addons:upgrade` command to migrate to a new plan.

````term
    $ heroku addons:upgrade s3search:newplan
    -----> Upgrading s3search:newplan to sharp-mountain-4005... done, v18 ($49/mo)
           Your plan has been updated to: s3search:newplan
````
## Removing the add-on

S3Search can be removed via the CLI.

<div class="warning" markdown="1">This will destroy all metadata and indexes stored in S3Search and cannot be undone! Of course, documents indexed in S3Search but stored elsewhere will remain untouched.</div>

````term
    $ heroku addons:remove s3search
    -----> Removing s3search from sharp-mountain-4005... done, v20 (free)
````
Before removing S3Search a data export can be performed by contacting support@s3searchapp.com directly.

## Support

All S3Search support and runtime issues should be submitted via on of the [Heroku Support channels](support-channels). Any non-support related issues or product feedback is welcome at feedback@s3searchapp.com.
