require 'integration_helper'

describe '/searches' do

  it 'is implemented correctly' do

    S3Search.get_all(per_page: 9999).map do |doc|
      S3Search.delete doc.id
    end

    baseline_query_count = S3Search.query_count

    title_a = SecureRandom.hex
    title_b = SecureRandom.hex
    
    a_docs = 10.times.map do
      {title: title_a}
    end

    b_docs = 10.times.map do
      {title: title_b}
    end

    a_doc_ids = S3Search.create_many(a_docs).map(&:id)
    b_doc_ids = S3Search.create_many(b_docs).map(&:id)
    expect(S3Search.query_count).to eq(baseline_query_count + 3)

    all_doc_ids = a_doc_ids + b_doc_ids

    S3Search.get_all(per_page: 20).each do |doc|
      expect(all_doc_ids).to include(doc.id)
    end
    expect(S3Search.query_count).to eq(baseline_query_count + 5)
    
    results = S3Search.simple_search "title:#{title_a}", per_page: 4
    expect(results.size).to eq(4)
    results.each do |doc|
      expect(a_doc_ids).to include(doc.id)
    end

    results.next
    expect(results.size).to eq(4)
    results.each do |doc|
      expect(a_doc_ids).to include(doc.id)
    end
    
    results.next
    expect(results.size).to eq(2)
    results.each do |doc|
      expect(a_doc_ids).to include(doc.id)
    end
    expect(results.next?).to be_false

    results = S3Search.simple_search "title:#{title_b}", per_page: 999
    expect(results.size).to eq(10)
    results.each do |doc|
      expect(b_doc_ids).to include(doc.id)
    end

    begin
      results.each do |result|
        S3Search.delete result.id
      end
    end while results.next
    
    expect(S3Search.query_count).to eq(baseline_query_count + 20)
  end
end
