module S3Search
  class ResultPage

    attr_reader :response, :results

    def initialize response, client
      @response = response
      @results = response.results
      @client = client
    end

    def next
      return unless next?
      change_page(links.next)
    end

    def next?
      links.next
    end

    def prev
      return unless prev?
      change_page(links.prev)
    end

    def prev?
      links.prev
    end

    def respond_to_missing? method_name, include_private=false
      results.respond_to?(method_name, include_private) || response.respond_to?(method_name, include_private)
    end

    private

    def change_page link
      @response = @client._get(link)
      @results = @response.results
      S3Search::ResultPage.new(@response, @client)
    end

    def method_missing method_name, *args, &block
      if results.respond_to?(method_name) 
        results.send method_name, *args, &block
      elsif response.respond_to?(method_name)
        response.send method_name, *args, &block
      else
        super
      end
    end

  end
end
